import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_dropzone/flutter_dropzone.dart';
import 'package:uuid/uuid.dart';
import 'package:velocity_components/constants/_constants.dart';
import 'package:velocity_components/widgets/icon_border.dart';

class ImageUploadView extends StatefulWidget {
  final storageRef;
  final double aspectRatio;
  final String imageUrl;
  final String directory;
  final Function(String) onImageUploaded;

  const ImageUploadView({
    Key key,
    @required this.storageRef,
    @required this.onImageUploaded,
    this.aspectRatio = 2,
    this.imageUrl,
    this.directory = 'images',
  }) : super(key: key);

  @override
  _ImageUploadViewState createState() => _ImageUploadViewState();
}

class _ImageUploadViewState extends State<ImageUploadView> {
  DropzoneViewController dropzoneViewController;
  Color borderColor = darkBlue;
  String imageUrl;
  bool loading = false;

  @override
  void initState() {
    imageUrl = widget.imageUrl;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () async {
        try {
          final file = (await dropzoneViewController.pickFiles())[0];
          var fileType = await dropzoneViewController.getFileMIME(file);
          if (fileType != 'image/jpeg' && fileType != 'image/png') {
            _showError('You can only upload .jpg and .png files');
            return;
          }

          this.setState(() {
            loading = true;
          });

          var storageRef = widget.storageRef
              .ref('${widget.directory}/${Uuid().v4()}.${fileType == 'image/png' ? 'png' : 'jpg'}');
          var uploadTaskSnapshot = await storageRef.put(file).future;

          Uri imageUri = await uploadTaskSnapshot.ref.getDownloadURL();

          String downloadUrl = imageUri.toString();
          widget.onImageUploaded(downloadUrl);

          this.setState(() {
            imageUrl = downloadUrl;
            loading = false;
          });
        } catch (e) {
          print(e);
        }
      },
      child: Stack(
        children: [
          Positioned.fill(
            child: DropzoneView(
              operation: DragOperation.copy,
              cursor: CursorType.grab,
              onCreated: (ctrl) => dropzoneViewController = ctrl,
              // onLoaded: () => print('Zone loaded'),
              onError: (ev) => print('Error: $ev'),
              onHover: () {
                this.setState(() {
                  borderColor = green;
                });
              },
              onDrop: (file) async {
                var fileType = await dropzoneViewController.getFileMIME(file);
                if (fileType != 'image/jpeg' && fileType != 'image/png') {
                  _showError('You can only upload .jpg and .png files');
                  return;
                }
                this.setState(() {
                  loading = true;
                });

                var storageRef = widget.storageRef.ref(
                    '${widget.directory}/${Uuid().v4()}.${fileType == 'image/png' ? 'png' : 'jpg'}');
                var uploadTaskSnapshot = await storageRef.put(file).future;

                Uri imageUri = await uploadTaskSnapshot.ref.getDownloadURL();

                String downloadUrl = imageUri.toString();
                widget.onImageUploaded(downloadUrl);

                this.setState(() {
                  borderColor = darkBlue;
                  imageUrl = downloadUrl;
                  loading = false;
                });
              },
              onLeave: () {
                this.setState(() {
                  borderColor = darkBlue;
                });
              },
            ),
          ),
          IconBorder(
            color: borderColor,
            icon: Icons.camera_alt_rounded,
            aspectRatio: widget.aspectRatio,
            child: loading
                ? Center(child: CircularProgressIndicator())
                : imageUrl != null
                    ? CachedNetworkImage(
                        imageUrl: imageUrl,
                        placeholder: (context, url) => Center(child: Icon(Icons.camera_alt)),
                        fit: BoxFit.cover,
                      )
                    : Center(child: Icon(Icons.camera_alt)),
          ),
        ],
      ),
    );
  }

  _showError(message) {
    final snackBar = SnackBar(content: Text(message));
    Scaffold.of(context).showSnackBar(snackBar);
  }
}
